@@ -3,6 +3,7 @@
 from typing import Optional, Tuple, List, Union
 
 import numpy
+import math
 import scipy.spatial
 
 from UM.Logger import Logger
@@ -13,7 +14,7 @@ class Polygon:
     """A class representing an immutable arbitrary 2-dimensional polygon."""
 
     @staticmethod
-    def approximatedCircle(radius):
+    def approximatedCircle(radius, num_segments = 8):
         """Return vertices from an approximate circle.
 
         An octagon is returned, which comes close enough to a circle.
@@ -22,19 +23,19 @@ class Polygon:
         :return: A polygon that approximates a circle.
         """
 
-        return Polygon(points = numpy.array([
-            [-radius, 0],
-            [-radius * 0.707, radius * 0.707],
-            [0, radius],
-            [radius * 0.707, radius * 0.707],
-            [radius, 0],
-            [radius * 0.707, -radius * 0.707],
-            [0, -radius],
-            [-radius * 0.707, -radius * 0.707]
-        ], numpy.float32))
+        step = 2 * math.pi / num_segments
+
+        points = []
+        for i in range(0, num_segments):
+            points.append([radius * -math.cos(i * step), radius * math.sin(i * step)])
+
+        return Polygon(points = numpy.array(points, numpy.float32))
 
     def __init__(self, points: Optional[Union[numpy.ndarray, List]] = None):
-        self._points = NumPyUtil.immutableNDArray(points)
+        if points is not None:
+            self._points = NumPyUtil.immutableNDArray(points)
+        else:
+            self._points = NumPyUtil.immutableNDArray([])
 
     def __eq__(self, other):
         if self is other:
@@ -63,7 +64,7 @@ class Polygon:
     def isValid(self) -> bool:
         return bool(self._points is not None and len(self._points) >= 3)
 
-    def getPoints(self) -> numpy.array:
+    def getPoints(self) -> numpy.ndarray:
         return self._points
 
     def project(self, normal) -> Tuple[float, float]:
@@ -133,6 +134,30 @@ class Polygon:
         point_matrix += point_on_axis
         return Polygon(point_matrix.getA()[::-1])
 
+    def scale(self, factor: float, origin: Optional[List[float]] = None) -> "Polygon":
+        """
+        Scales this polygon around a certain origin point.
+        :param factor: The scaling factor.
+        :param origin: Origin point around which to scale, 2D. As the scale
+        factor approaches 0, all coordinates will approach this origin point. As
+        the scale factor grows, all coordinates will move away from this origin
+        point. If `None`, the 0,0 coordinate will be used.
+        :return: A transformed polygon.
+        """
+        if origin is None:
+            origin = [0, 0]
+
+        transformation = numpy.identity(3) * factor  # Just the scaling matrix.
+        delta_scale = factor - 1
+        transformation[2][0] = delta_scale * -origin[0]
+        transformation[2][1] = delta_scale * -origin[1]
+
+        # Apply that affine transformation to the point data.
+        point_data = numpy.lib.pad(self._points, ((0, 0), (0, 1)), "constant", constant_values = (1))  # Turn 3D to do an affine transformation.
+        point_data = point_data.dot(transformation)
+
+        return Polygon(point_data[:, :-1])  # Leave out the affine component.
+
     def intersectionConvexHulls(self, other: "Polygon") -> "Polygon":
         """Computes the intersection of the convex hulls of this and another
         polygon.
