@@ -1,4 +1,4 @@
-# Copyright (c) 2019 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import imp
@@ -52,6 +52,8 @@ class PluginRegistry(QObject):
         PluginRegistry.__instance = self
 
         super().__init__(parent)
+        self.preloaded_plugins = []  # type: List[str]  # List of plug-in names that must be loaded before the rest, if the plug-ins are available. They are loaded in this order too.
+
         self._application = application  # type: Application
         self._api_version = application.getAPIVersion()  # type: Version
 
@@ -180,53 +182,6 @@ class PluginRegistry(QObject):
             is_in_path = False
         return is_in_path
 
-    def _verifyCleanHierarchy(self, abs_path: str) -> bool:
-        """ Only trust plugins if not checking or when there are no other files (not in the exceptions) in parent-root.
-        :param abs_path: The path to check for violations.
-        :return: True if the hierarchy of the abs_path and below can potentially be trusted.
-        """
-
-        # Don't check plugins if not in a 'security scenario'. Hobbyists must be able to tinker without signing.
-        if not self._check_if_trusted:
-            return True
-
-        install_prefix = os.path.abspath(self._application.getInstallPrefix())
-
-        # Put the parent-root on the work-list:
-        worklist = [abs_path]
-        while worklist:
-            current_dir = worklist.pop()
-
-            # If the directory under scrutiny is a signed folder or bundled, it's ok:
-            has_signature_file = os.path.isfile(os.path.join(current_dir, TrustBasics.getSignaturesLocalFilename()))
-            is_bundled = self._isPathInLocation(install_prefix, current_dir)
-            if has_signature_file or is_bundled:
-                continue
-
-            # Otherwise it's outside of the trusted area, and needs to be checked whether stray file/folder or plugin:
-            for file in os.listdir(current_dir):
-                abs_file = os.path.join(current_dir, file)
-
-                # If the file is a sub-folder, put it on the work-list to be investigated in a later iteration:
-                if os.path.isdir(abs_file):
-                    worklist.append(abs_file)
-
-                # Otherwise, the file can never have a valid signature associated with it, so message and abort:
-                else:
-                    Logger.error("Plugins in %s won't load: File that can't be verified: %s", abs_path, abs_file)
-                    if abs_path not in self._clean_hierarchy_sent_messages:
-                        self._clean_hierarchy_sent_messages.append(abs_path)
-                        message_text = i18n_catalog.i18nc("@error:untrusted",
-                                                          "Plugin {} was not loaded because it tried to load files outside of the trusted context",
-                                                          abs_path)
-                        # TODO: Message now has exactly the same string as for an unverified plugin, rather than a
-                        #       folder which contains other plugins, because of the string freeze in the current branch.
-                        Message(text=message_text).show()
-                    return False
-
-        # All is well:
-        return True
-
     # TODO:
     # - [ ] Improve how metadata is stored. It should not be in the 'plugin' prop
     #       of the dictionary item.
@@ -404,9 +359,17 @@ class PluginRegistry(QObject):
         """
 
         start_time = time.time()
+
+        # First load all of the pre-loaded plug-ins.
+        for preloaded_plugin in self.preloaded_plugins:
+            self.loadPlugin(preloaded_plugin)
+
         # Get a list of all installed plugins:
         plugin_ids = self._findInstalledPlugins()
         for plugin_id in plugin_ids:
+            if plugin_id in self.preloaded_plugins:
+                continue  # Already loaded this before.
+
             # Get the plugin metadata:
             try:
                 plugin_metadata = self.getMetaData(plugin_id)
@@ -575,7 +538,10 @@ class PluginRegistry(QObject):
             del self.bundled_plugin_cache[plugin_id]
 
         Logger.log("i", "Attempting to remove plugin '%s' from directory '%s'", plugin_id, plugin_path)
-        shutil.rmtree(plugin_path)
+        try:
+            shutil.rmtree(plugin_path)
+        except EnvironmentError as e:
+            Logger.error("Unable to remove plug-in {plugin_id}: {err}".format(plugin_id = plugin_id, err = str(e)))
 
 #===============================================================================
 # PRIVATE METHODS
@@ -631,19 +597,35 @@ class PluginRegistry(QObject):
 
         if plugin_id in self._found_plugins:
             return self._found_plugins[plugin_id]
-        location = None
+        locations = []
         for folder in self._plugin_locations:
-            if not self._verifyCleanHierarchy(folder):
-                continue
             location = self._locatePlugin(plugin_id, folder)
             if location:
-                break
+                locations.append(location)
 
-        if not location:
+        if not locations:
             return None
+        final_location = locations[0]
 
+        if len(locations) > 1:
+            # We found multiple versions of the plugin. Let's find out which one to load!
+            highest_version = Version(0)
+
+            for loc in locations:
+                meta_data = {}  # type: Dict[str, Any]
+                plugin_location = os.path.join(loc, plugin_id)
+                metadata_file = os.path.join(plugin_location, "plugin.json")
+                try:
+                    with open(metadata_file, "r", encoding="utf-8") as file_stream:
+                        self._parsePluginInfo(plugin_id, file_stream.read(), meta_data)
+                except:
+                    pass
+                current_version = Version(meta_data["plugin"]["version"])
+                if current_version > highest_version:
+                    highest_version = current_version
+                    final_location = loc
         try:
-            file, path, desc = imp.find_module(plugin_id, [location])
+            file, path, desc = imp.find_module(plugin_id, [final_location])
         except Exception:
             Logger.logException("e", "Import error when importing %s", plugin_id)
             return None
