@@ -3,7 +3,7 @@
 
 from PyQt5.QtQml import qmlRegisterType, qmlRegisterSingletonType, qmlRegisterUncreatableType
 
-from UM.Qt.Bindings import StageModel
+from UM.Qt.Bindings import StageModel, FileProviderModel, ProjectOutputDevicesModel
 from UM.Qt.Duration import Duration, DurationFormat
 
 from . import MainWindow
@@ -112,6 +112,10 @@ class Bindings:
         # Additions after 4.6
         qmlRegisterSingletonType(Utilities.UrlUtil, "UM", 1, 5, "UrlUtil", Utilities.createUrlUtil)
 
+        # Additions after 4.9
+        qmlRegisterType(FileProviderModel.FileProviderModel, "UM", 1, 6, "FileProviderModel")
+        qmlRegisterType(ProjectOutputDevicesModel.ProjectOutputDevicesModel, "UM", 1, 6, "ProjectOutputDevicesModel")
+
     @staticmethod
     def addRegisterType(class_type: type, qml_import_name: str, major_version: int, minor_version: int, class_name: str) -> None:
         qmlRegisterType(class_type, qml_import_name, major_version, minor_version, class_name)
