@@ -4,7 +4,8 @@
 import base64
 import json
 import os
-from typing import Callable, Optional, Tuple
+from pathlib import Path
+from typing import Callable, Dict, Optional, Tuple, List, cast
 
 # Note that we unfortunately need to use 'hazmat' code, as there apparently is no way to do what we want otherwise.
 # (Even if what we want should be relatively commonplace in security.)
@@ -32,6 +33,7 @@ class TrustBasics:
     # For (in) directories (plugins for example):
     __signatures_relative_filename = "signature.json"
     __root_signatures_category = "root_signatures"
+    __root_signed_manifest_key = "root_manifest_signature"
 
     # For(/next to) single files:
     __signature_filename_extension = ".signature"
@@ -71,6 +73,15 @@ class TrustBasics:
 
         return cls.__root_signatures_category
 
+    @classmethod
+    def getRootSignedManifestKey(cls) -> str:
+        """'Signed folder' scenario: This is the (json-)key for the hash that (self-)signs the signing file.
+
+        :return: The json 'name' for the key that contains the signature that signs all others' in the file.
+        """
+
+        return cls.__root_signed_manifest_key
+
     @classmethod
     def getSignaturePathForFile(cls, filename: str) -> str:
         """'Single signed file' scenario: Get the name of the signature-file that should be located next to the file.
@@ -125,6 +136,45 @@ class TrustBasics:
             Logger.logException("e", "Couldn't read '{0}' for plain hash generation.".format(filename))
         return None
 
+    @classmethod
+    def getSelfSignHash(cls, signatures: Dict[str, str]) -> Optional[str]:
+        """ Make a hash for the signature 'file' itself, where the file is represented as a dictionary here.
+
+        :param signatures: A dictionary of (filename, file-hash-signature) pairs.
+        :return: A hash, which can be used for creating or checking against a self-signed manifest.
+        """
+
+        concat = "".join([str(key) + str(signatures.get(key)) for key in sorted(signatures.keys())])
+        if concat is None or len(concat) < 1:
+            return None
+        hasher = hashes.Hash(cls.__hash_algorithm, backend=default_backend())
+        hasher.update(concat.encode())
+        return base64.b64encode(hasher.finalize()).decode("utf-8")
+
+    @classmethod
+    def getHashSignature(cls, shash: str, private_key: RSAPrivateKey, err_info: Optional[str] = None) -> Optional[str]:
+        """ Creates the signature for the provided hash, given a private key.
+
+        :param shash: The provided string.
+        :param private_key: The private key used for signing.
+        :param err_info: Some optional extra info to be printed on error (for ex.: a filename the data came from).
+        :return: The signature if successful, 'None' otherwise.
+        """
+
+        try:
+            hash_bytes = base64.b64decode(shash)
+            signature_bytes = private_key.sign(
+                hash_bytes,
+                padding.PSS(mgf=padding.MGF1(cls.__hash_algorithm), salt_length=padding.PSS.MAX_LENGTH),
+                Prehashed(cls.__hash_algorithm)
+            )
+            return base64.b64encode(signature_bytes).decode("utf-8")
+        except:  # Yes, we  do really want this on _every_ exception that might occur.
+            if err_info is None:
+                err_info = "HASH:" + shash
+            Logger.logException("e", "Couldn't sign '{0}', no signature generated.".format(err_info))
+        return None
+
     @classmethod
     def getFileSignature(cls, filename: str, private_key: RSAPrivateKey) -> Optional[str]:
         """Creates the signature for the (hash of the) provided file, given a private key.
@@ -137,17 +187,7 @@ class TrustBasics:
         file_hash = cls.getFileHash(filename)
         if file_hash is None:
             return None
-        try:
-            file_hash_bytes = base64.b64decode(file_hash)
-            signature_bytes = private_key.sign(
-                file_hash_bytes,
-                padding.PSS(mgf = padding.MGF1(cls.__hash_algorithm), salt_length = padding.PSS.MAX_LENGTH),
-                Prehashed(cls.__hash_algorithm)
-            )
-            return base64.b64encode(signature_bytes).decode("utf-8")
-        except:  # Yes, we  do really want this on _every_ exception that might occur.
-            Logger.logException("e", "Couldn't sign '{0}', no signature generated.".format(filename))
-        return None
+        return cls.getHashSignature(file_hash, private_key, filename)
 
     @staticmethod
     def generateNewKeyPair() -> Tuple[RSAPrivateKeyWithSerialization, RSAPublicKey]:
@@ -172,7 +212,7 @@ class TrustBasics:
             password_bytes = None if optional_password is None else optional_password.encode()
             with open(private_filename, "rb") as file:
                 private_key = load_pem_private_key(file.read(), backend=default_backend(), password=password_bytes)
-                return private_key
+                return cast(RSAPrivateKey, private_key)
         except:  # Yes, we  do really want this on _every_ exception that might occur.
             Logger.logException("e", "Couldn't load private-key.")
         return None
@@ -217,16 +257,23 @@ class TrustBasics:
     @staticmethod
     def removeCached(path: str) -> bool:
         try:
-            cache_folders_to_empty = []  # List[str]
+            cache_folders_to_empty = []  # type: List[str]
+            cache_files_to_remove = []  # type: List[str]
             for root, dirnames, filenames in os.walk(path, followlinks=True):
                 for dirname in dirnames:
                     if dirname == "__pycache__":
                         cache_folders_to_empty.append(os.path.join(root, dirname))
+                for filename in filenames:
+                    if Path(filename).suffix == ".qmlc":
+                        cache_files_to_remove.append(os.path.join(root, filename))
             for cache_folder in cache_folders_to_empty:
                 for root, dirnames, filenames in os.walk(cache_folder, followlinks=True):
                     for filename in filenames:
                         os.remove(os.path.join(root, filename))
+            for cache_file in cache_files_to_remove:
+                os.remove(cache_file)
             return True
+
         except:  # Yes, we  do really want this on _every_ exception that might occur.
             Logger.logException("e", "Removal of pycache for unbundled path '{0}' failed.".format(path))
         return False
@@ -288,9 +335,11 @@ class Trust:
         """
 
         self._public_key = None  # type: Optional[RSAPublicKey]
+        self._follow_symlinks = False  # type: bool
+
         try:
             with open(public_key_filename, "rb") as file:
-                self._public_key = load_pem_public_key(file.read(), backend = default_backend())
+                self._public_key = cast(RSAPublicKey, load_pem_public_key(file.read(), backend = default_backend()))
         except:  # Yes, we  do really want this on _every_ exception that might occur.
             self._public_key = None
             raise Exception("e", "Couldn't load public-key '{0}'.".format(public_key_filename))
@@ -303,26 +352,31 @@ class Trust:
 
         self._violation_handler = violation_handler  # type: Callable[[str], None]
 
-    def _verifyFile(self, filename: str, signature: str) -> bool:
+    def _verifyHash(self, shash: str, signature: str, err_info: Optional[str] = None) -> bool:
         if self._public_key is None:
             return False
-        file_hash = TrustBasics.getFileHash(filename)
-        if file_hash is None:
-            return False
         try:
             signature_bytes = base64.b64decode(signature)
-            file_hash_bytes = base64.b64decode(file_hash)
+            hash_bytes = base64.b64decode(shash)
             self._public_key.verify(
                 signature_bytes,
-                file_hash_bytes,
+                hash_bytes,
                 padding.PSS(mgf = padding.MGF1(TrustBasics.getHashAlgorithm()), salt_length = padding.PSS.MAX_LENGTH),
                 Prehashed(TrustBasics.getHashAlgorithm())
             )
             return True
         except:  # Yes, we  do really want this on _every_ exception that might occur.
-            self._violation_handler("Couldn't verify '{0}' with supplied signature.".format(filename))
+            if err_info is None:
+                err_info = "HASH:" + shash
+            self._violation_handler("Couldn't verify '{0}' with supplied signature.".format(err_info))
         return False
 
+    def _verifyFile(self, filename: str, signature: str) -> bool:
+        file_hash = TrustBasics.getFileHash(filename)
+        if file_hash is None:
+            return False
+        return self._verifyHash(file_hash, signature, filename)
+
     def signedFolderCheck(self, path: str) -> bool:
         """In the 'singed folder' case, check whether the folder is signed according to the Trust-objects' public key.
 
@@ -338,17 +392,33 @@ class Trust:
                 json_data = json.load(data_file)
                 signatures_json = json_data.get(TrustBasics.getRootSignatureCategory(), None)
                 if signatures_json is None:
-                    Logger.logException("e", "Can't parse (folder) signature file '{0}'.".format(data_file))
+                    self._violation_handler("Can't parse (folder) signature file '{0}'.".format(data_file))
+                    return False
+
+                # Any filename outside of the plugin-root is a sure sign of tampering:
+                for key in signatures_json.keys():
+                    if ".." in key:
+                        self._violation_handler("Suspect key '{0}' in signature file '{1}'.".format(key, data_file))
+                        return False
+
+                # Check if the signing file itself has been tampered with:
+                self_sign = json_data.get(TrustBasics.getRootSignedManifestKey(), None)
+                if self_sign is None:
+                    self._violation_handler("Signature file '{0}' is not a self-signed manifest.".format(data_file))
+                    return False
+                shash = TrustBasics.getSelfSignHash(signatures_json)
+                if shash is None or not self._verifyHash(shash, self_sign):
+                    self._violation_handler("Suspect self-sign in signature-file '{0}'.".format(data_file))
                     return False
 
                 # Loop over all files within the folder (excluding the signature file):
                 file_count = 0
-                for root, dirnames, filenames in os.walk(path, followlinks = True):
+                for root, dirnames, filenames in os.walk(path, followlinks = self._follow_symlinks):
                     for filename in filenames:
                         if filename == TrustBasics.getSignaturesLocalFilename() and root == path:
                             continue
-                        file_count += 1
                         name_on_disk, name_in_data = TrustBasics.getFilePathInfo(path, root, filename)
+                        file_count += 1
 
                         # Get the signature for the current to-verify file:
                         signature = signatures_json.get(name_in_data, None)
@@ -360,6 +430,10 @@ class Trust:
                         if not self._verifyFile(name_on_disk, signature):
                             self._violation_handler("File '{0}' didn't match with checksum.".format(name_on_disk))
                             return False
+                    for dirname in dirnames:
+                        dir_full_path = os.path.join(path, dirname)
+                        if os.path.islink(dir_full_path) and not self._follow_symlinks:
+                            Logger.log("w", "Directory symbolic link '{0}' will not be followed.".format(dir_full_path))
 
                 # The number of correctly signed files should be the same as the number of signatures:
                 if len(signatures_json.keys()) != file_count:
@@ -411,6 +485,9 @@ class Trust:
 
         return os.path.exists(TrustBasics.getSignaturePathForFile(filename))
 
+    def setFollowSymlinks(self, follow_symlinks: bool) -> None:
+        self._follow_symlinks = follow_symlinks
+
 
 class TrustException(Exception):
     pass
