@@ -15,27 +15,31 @@ from UM.Mesh.MeshWriter import MeshWriter
 from UM.Message import Message
 from UM.OutputDevice import OutputDeviceError
 from UM.OutputDevice.OutputDevice import OutputDevice
+from UM.OutputDevice.ProjectOutputDevice import ProjectOutputDevice
 from UM.i18n import i18nCatalog
 
 catalog = i18nCatalog("uranium")
 
 
-class LocalFileOutputDevice(OutputDevice):
+class LocalFileOutputDevice(ProjectOutputDevice):
     """Implements an OutputDevice that supports saving to arbitrary local files."""
 
-    def __init__(self):
-        super().__init__("local_file")
+    def __init__(self, add_to_output_devices: bool = True, parent = None):
+        super().__init__(device_id = "local_file", add_to_output_devices = add_to_output_devices, parent = parent)
 
         self.setName(catalog.i18nc("@item:inmenu", "Local File"))
-        self.setShortDescription(catalog.i18nc("@action:button Preceded by 'Ready to'.", "Save to File"))
-        self.setDescription(catalog.i18nc("@info:tooltip", "Save to File"))
+        self.setShortDescription(catalog.i18nc("@action:button Preceded by 'Ready to'.", "Save to Disk"))
+        self.setDescription(catalog.i18nc("@info:tooltip", "Save to Disk"))
         self.setIconName("save")
 
+        self.shortcut = "Ctrl+S"
+        self.menu_entry_text = "To Disk"
+
         self._writing = False
 
     def requestWrite(self, nodes, file_name = None, limit_mimetypes = None, file_handler = None, **kwargs):
         """Request the specified nodes to be written to a file.
-        
+
         :param nodes: A collection of scene nodes that should be written to the
         file.
         :param file_name: A suggestion for the file name to write
@@ -51,7 +55,7 @@ class LocalFileOutputDevice(OutputDevice):
         # Set up and display file dialog
         dialog = QFileDialog()
 
-        dialog.setWindowTitle(catalog.i18nc("@title:window", "Save to File"))
+        dialog.setWindowTitle(catalog.i18nc("@title:window", "Save to Disk"))
         dialog.setFileMode(QFileDialog.AnyFile)
         dialog.setAcceptMode(QFileDialog.AcceptSave)
 
@@ -106,7 +110,8 @@ class LocalFileOutputDevice(OutputDevice):
 
         # CURA-6411: This code needs to be before dialog.selectFile and the filters, because otherwise in macOS (for some reason) the setDirectory call doesn't work.
         stored_directory = Application.getInstance().getPreferences().getValue("local_file/dialog_save_path")
-        dialog.setDirectory(stored_directory)
+        if stored_directory and stored_directory != "":
+            dialog.setDirectory(stored_directory)
 
         # Add the file name before adding the extension to the dialog
         if file_name is not None:
@@ -128,7 +133,7 @@ class LocalFileOutputDevice(OutputDevice):
         # Get file name from file dialog
         file_name = dialog.selectedFiles()[0]
         Logger.log("d", "Writing to [%s]..." % file_name)
-        
+
         if os.path.exists(file_name):
             result = QMessageBox.question(None, catalog.i18nc("@title:window", "File Already Exists"), catalog.i18nc("@label Don't translate the XML tag <filename>!", "The file <filename>{0}</filename> already exists. Are you sure you want to overwrite it?").format(file_name))
             if result == QMessageBox.No:
@@ -172,7 +177,7 @@ class LocalFileOutputDevice(OutputDevice):
             raise OutputDeviceError.PermissionDeniedError(catalog.i18nc("@info:status Don't translate the XML tags <filename>!", "Permission denied when trying to save <filename>{0}</filename>").format(file_name)) from e
         except OSError as e:
             Logger.log("e", "Operating system would not let us write to %s: %s", file_name, str(e))
-            raise OutputDeviceError.WriteRequestFailedError(catalog.i18nc("@info:status Don't translate the XML tags <filename> or <message>!", "Could not save to <filename>{0}</filename>: <message>{1}</message>").format()) from e
+            raise OutputDeviceError.WriteRequestFailedError(catalog.i18nc("@info:status Don't translate the XML tags <filename> or <message>!", "Could not save to <filename>{0}</filename>: <message>{1}</message>").format(file_name, str(e))) from e
 
     def _onJobProgress(self, job, progress):
         self.writeProgress.emit(self, progress)
