@@ -1,10 +1,11 @@
-# Copyright (c) 2019 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 from typing import List, Tuple, TYPE_CHECKING, Optional
 
 from PyQt5.QtCore import Qt
 
 from UM.Event import Event, MouseEvent, KeyEvent
+from UM.Logger import Logger
 from UM.Math.Float import Float
 from UM.Math.Matrix import Matrix
 from UM.Math.Plane import Plane
@@ -294,7 +295,7 @@ class ScaleTool(Tool):
 
         if Selection.hasSelection():
             ## Ensure that the returned value is positive (mirror causes scale to be negative)
-            return abs(round(float(Selection.getSelectedObject(0).getScale().x), 4))
+            return abs(round(float(self._getScaleInWorldCoordinates(Selection.getSelectedObject(0)).x), 4))
 
         return 1.0
 
@@ -306,7 +307,7 @@ class ScaleTool(Tool):
 
         if Selection.hasSelection():
             ## Ensure that the returned value is positive (mirror causes scale to be negative)
-            return abs(round(float(Selection.getSelectedObject(0).getScale().y), 4))
+            return abs(round(float(self._getScaleInWorldCoordinates(Selection.getSelectedObject(0)).y), 4))
 
         return 1.0
 
@@ -318,7 +319,7 @@ class ScaleTool(Tool):
 
         if Selection.hasSelection():
             ## Ensure that the returned value is positive (mirror causes scale to be negative)
-            return abs(round(float(Selection.getSelectedObject(0).getScale().z), 4))
+            return abs(round(float(self._getScaleInWorldCoordinates(Selection.getSelectedObject(0)).z), 4))
 
         return 1.0
 
@@ -386,7 +387,7 @@ class ScaleTool(Tool):
 
         obj = Selection.getSelectedObject(0)
         if obj:
-            obj_scale = obj.getScale()
+            obj_scale = self._getScaleInWorldCoordinates(obj)
             if round(float(obj_scale.x), 4) != scale:
                 scale_factor = abs(scale / obj_scale.x)
                 if self._non_uniform_scale:
@@ -404,7 +405,7 @@ class ScaleTool(Tool):
 
         obj = Selection.getSelectedObject(0)
         if obj:
-            obj_scale = obj.getScale()
+            obj_scale = self._getScaleInWorldCoordinates(obj)
             if round(float(obj_scale.y), 4) != scale:
                 scale_factor = abs(scale / obj_scale.y)
                 if self._non_uniform_scale:
@@ -422,7 +423,7 @@ class ScaleTool(Tool):
 
         obj = Selection.getSelectedObject(0)
         if obj:
-            obj_scale = obj.getScale()
+            obj_scale = self._getScaleInWorldCoordinates(obj)
             if round(float(obj_scale.z), 4) != scale:
                 scale_factor = abs(scale / obj_scale.z)
                 if self._non_uniform_scale:
@@ -433,6 +434,8 @@ class ScaleTool(Tool):
                 self._scaleSelectedNodes(scale_vector)
 
     def _scaleSelectedNodes(self, scale_vector: Vector) -> None:
+        if scale_vector.length() == 0.0:
+            return
         selected_nodes = self._getSelectedObjectsWithoutSelectedAncestors()
         if len(selected_nodes) > 1:
             op = GroupedOperation()
@@ -463,10 +466,26 @@ class ScaleTool(Tool):
     def _getSVDRotationFromMatrix(self, matrix):
         result = Matrix()
         rotation_data = matrix.getData()[:3, :3]
-        U, s, Vh = scipy.linalg.svd(rotation_data)
-        result._data[:3, :3] = U.dot(Vh)
+        try:
+            U, s, Vh = scipy.linalg.svd(rotation_data)
+            result._data[:3, :3] = U.dot(Vh)
+        except ValueError as ex:
+            Logger.log("e", "Could not perform SVD for matrix {} because {}.".format(str(rotation_data), str(ex)))
         return result
 
+    def _getExtents(self, node, matrix):
+        extents = None
+        modified_matrix = matrix.multiply(node.getLocalTransformation(), copy = True)
+        if node.getMeshData():
+            extents = node.getMeshData().getExtents(modified_matrix)
+
+        for child in node.getChildren():
+            if extents is None:
+                extents = self._getExtents(child, modified_matrix)
+            else:
+                extents = extents + self._getExtents(child, modified_matrix)
+        return extents
+
     def _getRotatedExtents(self, node, with_translation = False):
         # The rotation matrix that we get back from our own decompose isn't quite correct for some reason.
         # It seems that it does not "draw the line" between scale, rotate & skew quite correctly in all cases.
@@ -477,16 +496,17 @@ class ScaleTool(Tool):
         # In order to remedy this, we use singular value decomposition.
         # SVD solves a = U s V.H for us, where A is the matrix. U and V.h are Rotation matrices and s holds the scale.
         extents = None
+        rotated_matrix = self._getSVDRotationFromMatrix(node.getWorldTransformation())
         if node.getMeshData():
-            rotated_matrix = self._getSVDRotationFromMatrix(node.getWorldTransformation())
             if with_translation:
                 rotated_matrix._data[:3, 3] = node.getPosition().getData()
 
             extents = node.getMeshData().getExtents(rotated_matrix)
-        for child in node.getChildren():
-            # We want the children with their (local) translation, as this influences the size of the AABB.
-            if extents is None:
-                extents = self._getRotatedExtents(child, with_translation = True)
-            else:
-                extents = extents + self._getRotatedExtents(child, with_translation = True)
+        if node.callDecoration("isGroup"):
+            for child in node.getChildren():
+                # We want the children with their (local) translation, as this influences the size of the AABB.
+                if extents is None:
+                    extents = self._getExtents(child, rotated_matrix)
+                else:
+                    extents = extents + self._getExtents(child, rotated_matrix)
         return extents
\ No newline at end of file
